#include "GetPoint.js"

/*
_IonicVolute script by Brian McMillin March 30, 2016
This script is based upon the paper by Denise Andrey and Mirko Galli, specifically
their equation derived from work by Dürer (1525), Giullaume Philandrier (1544). 
http://link.springer.com/article/10.1007%2Fs00004-004-0017-4#/page-1
http://www.springerlink.com/content/r61qh2354m56qt31/fulltext.pdf
http://www.emis.ams.org/journals/NNJ/AndGal.html
http://www.c4dcafe.com/ipb/forums/topic/59105-ionic-volute/
There are good pictures of an Ionic Volute here:
https://en.wikipedia.org/wiki/Ionic_order
Their generalized equation was generalized even more for MoI.


This is a 2D script.  z = 0.0 plus the z value of selected center point.
Default parameters, based upon actual Ionic volutes, and assuming the origin is the center point selected:
The spiral has 3 whorls (turns).
The eye circle has a diameter one unit.
The spiral begins at the top of the eye, at (0, 0.5).
The spiral is made up of 3 whorls, and the top of the spiral is at (0, 4.5).
This value of 4.5 units is the default "growth" parameter.
The default "offset" parameter of 3.5 units is calculated to be (0, -3.5), near the bottom of the spiral,
but actually is at about -3.4714 units.  (Due to continuour formula?)
The minimum y value of the default spiral is about -3.482932 units.  
Angle theta ranges from PI/2 to (whorls*2*PI + 1)/2, which is 13*PI/2 for 3 whorls.

The spiral can also be made clockwise, rotated at an angle, or spun 180 degrees.
Changing only the offset parameter, for a second spiral, creates a new spiral that is offset
from the original spiral, but the pointy ends coincide.  Joining these two spirals makes an interesting surface.
The joined spirals can be filleted, or extruded and filleted.

The user may do further modeling in MoI.  

Polar coordinates are used, which consist of an angle theta (alpha in paper), and radius r.
The equation is radius = offset * tan(beta + (theta - PI/2)/(whorls*2*PI) * (gamma - beta)
Polar coordinates are converted to Cartesian.
*/

function DoIonicVolute()
{
	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) ) return;

	var centerPt = pointpicker.pt;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'Options' );
	moi.ui.endUIUpdate();
	
	var factory = moi.ui.commandUI.Update(centerPt);

	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			factory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'whorls' || dlg.event == 'numpoints' || dlg.event == 'offset' || dlg.event == 'offsetslider' || dlg.event == 'growth' || dlg.event == 'growthslider' || dlg.event == 'eyeRadius' || dlg.event == 'eyeslider' || dlg.event == 'startAngle' || dlg.event == 'angleslider' || dlg.event == 'clockwise' || dlg.event == 'spinit'  )
		{
			factory.cancel();
			factory = moi.ui.commandUI.Update(centerPt);
			moi.ui.commandUI.updating = false;
		}
	}
	factory.commit();
}

DoIonicVolute();
